@@ -111,7 +111,11 @@ Dictionary dictionary_create_lang(const char * lang)
 	/* If an sql database exists, try to read that. */
 	if (check_db(lang))
 	{
+#if HAVE_SQLITE3
 		dictionary = dictionary_create_from_db(lang);
+#else
+		return NULL;
+#endif /* HAVE_SQLITE3 */
 	}
 
 	/* Fallback to a plain-text dictionary */
@@ -338,7 +342,6 @@ void dictionary_delete(Dictionary dict)
 	string_id_delete(dict->define.set);
 	free(dict->define.name);
 	free(dict->define.value);
-	free((void *)dict->suppress_warning);
 	free_regexs(dict->regex_root);
 	free_anysplit(dict);
 	free_dictionary(dict);
@@ -370,9 +373,6 @@ bool dictionary_generation_request(const Dictionary dict)
 	const char *generation_mode = test_enabled("generate");
 	if (generation_mode != NULL)
 	{
-		const size_t initial_allocation = 256;
-		dict->num_categories_alloced = initial_allocation;
-		dict->category = malloc(sizeof(*dict->category) *initial_allocation);
 		dict->generate_walls =
 			feature_enabled(generation_mode, "walls", NULL) != NULL;
 		dict->spell_checker = NULL; /* Disable spell-checking. */
