@@ -16,6 +16,7 @@
 
 #include "api-types.h"                  // pp_knowledge
 #include "connectors.h"                 // ConTable
+#include "dict-defines.h"
 #include "dict-structures.h"
 #include "memory-pool.h"                // Pool_desc
 #include "utilities.h"                  // locale_t
@@ -25,6 +26,15 @@
 #define LIMITED_CONNECTORS_WORD ("LENGTH-LIMIT-")
 #define IS_GENERATION(dict) (dict->category != NULL)
 
+/* If the maximum disjunct cost is yet uninitialized, the value defined in the
+ * dictionary (or if not defined then DEFAULT_MAX_DISJUNCT_COST) is used. */
+static const double UNINITIALIZED_MAX_DISJUNCT_COST = -10000.0;
+static const double DEFAULT_MAX_DISJUNCT_COST = 2.7;
+/* We need some of these as literal strings. */
+#define LG_DISJUNCT_COST                        "max-disjunct-cost"
+#define LG_DICTIONARY_VERSION_NUMBER            "dictionary-version-number"
+#define LG_DICTIONARY_LOCALE                    "dictionary-locale"
+
 /* Forward decls */
 typedef struct Afdict_class_struct Afdict_class;
 typedef struct Regex_node_s Regex_node;
@@ -41,7 +51,7 @@ struct X_node_struct
 /* The regexes are stored as a linked list of the following nodes. */
 struct Regex_node_s
 {
-	char *name;      /* The identifying name of the regex */
+	const char *name;/* The identifying name of the regex */
 	char *pattern;   /* The regular expression pattern */
 	bool neg;        /* Negate the match */
 	void *re;        /* The compiled regex. void * to avoid
@@ -150,7 +160,6 @@ struct Dictionary_s
 	const char    * input;
 	const char    * pin;
 	bool            recursive_error;
-	const char    * suppress_warning;
 	bool            is_special;
 	int             already_got_it; /* For char, but needs to hold EOF */
 	int             line_number;
@@ -174,4 +183,9 @@ bool dictionary_generation_request(const Dictionary);
 bool dict_has_word(const Dictionary dict, const char *);
 void add_empty_word(Sentence, X_node *);
 
+static inline const char *subscript_mark_str(void)
+{
+	static const char sm[] = { SUBSCRIPT_MARK, '\0' };
+	return sm;
+}
 #endif /* _LG_DICT_COMMON_H_ */
