@@ -304,8 +304,7 @@ const char * linkgrammar_get_dict_version(Dictionary dict)
 		return dict->version;
 	}
 
-	/* Original code is left for backward compatibility. Note that the
-	 * check for now version should be moved up if it is ever removed. */
+	/* Original code is left for backward compatibility. */
 
 	char * ver;
 	char * p;
@@ -693,8 +692,6 @@ bool afdict_init(Dictionary dict)
 	ac = AFCLASS(afdict, AFDICT_SANEMORPHISM);
 	if (0 != ac->length)
 	{
-		int rc;
-
 		Regex_node *sm_re = malloc(sizeof(*sm_re));
 		dyn_str *rebuf = dyn_str_new();
 
@@ -716,15 +713,16 @@ bool afdict_init(Dictionary dict)
 		dyn_str_delete(rebuf);
 
 		afdict->regex_root = sm_re;
-		sm_re->name = strdup(afdict_classname[AFDICT_SANEMORPHISM]);
+		sm_re->name = string_set_add(afdict_classname[AFDICT_SANEMORPHISM],
+		                             afdict->string_set);
 		sm_re->re = NULL;
 		sm_re->next = NULL;
 		sm_re->neg = false;
-		rc = compile_regexs(afdict->regex_root, afdict);
-		if (rc) {
+		if (!compile_regexs(afdict->regex_root, afdict))
+		{
 			prt_error("Error: afdict_init: Failed to compile "
-			          "regex '%s' in file %s, return code %d\n",
-			          afdict_classname[AFDICT_SANEMORPHISM], afdict->name, rc);
+			          "regex /%s/ in file \"%s\"\n",
+			          afdict_classname[AFDICT_SANEMORPHISM], afdict->name);
 			return false;
 		}
 		lgdebug(+D_AI, "%s regex %s\n",
@@ -769,12 +767,12 @@ bool afdict_init(Dictionary dict)
 		for (ac = afdict->afdict_class;
 		     ac < &afdict->afdict_class[ARRAY_SIZE(afdict_classname)]; ac++)
 		{
-				if (0 == ac->length) continue;
-				lgdebug(+0, "Class %s, %zu items:",
-				        afdict_classname[ac-afdict->afdict_class], ac->length);
-				for (l = 0; l < ac->length; l++)
-					lgdebug(0, " '%s'", ac->string[l]);
-				lgdebug(0, "\n\\");
+			if (0 == ac->length) continue;
+			lgdebug(+0, "Class %s, %zu items:",
+			        afdict_classname[ac-afdict->afdict_class], ac->length);
+			for (l = 0; l < ac->length; l++)
+				lgdebug(0, " '%s'", ac->string[l]);
+			lgdebug(0, "\n\\");
 		}
 		lg_error_flush();
 	}
