@@ -114,6 +114,8 @@ dictionary_six_str(const char * lang,
 	dict = (Dictionary) malloc(sizeof(struct Dictionary_s));
 	memset(dict, 0, sizeof(struct Dictionary_s));
 
+	dict->line_number = 1;
+
 	/* Language and file-name stuff */
 	dict->string_set = string_set_create();
 	t = find_last_dir_separator((char *)lang);
@@ -124,8 +126,16 @@ dictionary_six_str(const char * lang,
 
 	if (NULL != affix_name)
 	{
-		if (!dictionary_generation_request(dict))
+		if (dictionary_generation_request(dict))
+		{
+			const size_t initial_allocation = 256;
+			dict->num_categories_alloced = initial_allocation;
+			dict->category = malloc(sizeof(*dict->category) * initial_allocation);
+		}
+		else
+		{
 			dict->spell_checker = spellcheck_create(dict->lang);
+		}
 
 #if defined HAVE_HUNSPELL || defined HAVE_ASPELL
 		/* FIXME: Move to spellcheck-*.c */
@@ -224,7 +234,7 @@ dictionary_six_str(const char * lang,
 	setlocale(LC_CTYPE, dict->locale);
 	lgdebug(+D_DICT, "Regexs locale \"%s\"\n", setlocale(LC_CTYPE, NULL));
 
-	if (compile_regexs(dict->regex_root, dict))
+	if (!compile_regexs(dict->regex_root, dict))
 	{
 		locale = setlocale(LC_CTYPE, locale);         /* Restore the locale. */
 		assert(NULL != locale, "Cannot restore program locale");
