@@ -14,6 +14,7 @@
 #include <string.h>
 
 #include "api-structures.h"             // Sentence_s (add_empty_word)
+#include "connectors.h"
 #include "dict-common/dialect.h"
 #include "dict-common/dict-affix.h"     // is_stem
 #include "dict-common/dict-common.h"
@@ -83,8 +84,11 @@
 
   A number following a square bracket over-rides the cost of that bracket.
   Thus, [...].5 has a cost of 0.5 while [...]2.0 has a cost of 2; that
-  is it is the same as [[...]].  Any floating point number (including
-  exponents!) is allowed.
+  is it is the same as [[...]].  Any floating point number is allowed.
+
+  Instead of a numerical cost, a symbolic cost can be used, a.k.a. a
+  "dialect component name".  The file "4.0.dialect" defines dialect names
+  (in square brackets) and their component names and values.
 
   The expression "(A+ or ())" means that you can choose either "A+" or
   the empty expression "()", that is, that the connector "A+" is
@@ -99,8 +103,8 @@
   used.
 
   The dictionary writer is not allowed to use connectors that begin in
-  "ID".  This is reserved for the connectors automatically
-  generated for idioms.
+  "_".  This is reserved for connectors that are automatically
+  generated (currently only for idioms).
 
   Dictionary words may be followed by a dot (period, "."), and a "subscript"
   identifying the word type. The subscript may be one or more letters or
@@ -108,6 +112,9 @@
   (mostly?) subscripts consisting of a single letter, and these serve mostly
   to identify the part-of-speech. In general, subscripts can also be used
   to distinguish different word senses.
+
+  Subscripts that start with "_" are reserved for words that are
+  automatically generated (currently only for idioms).
 */
 
 static bool link_advance(Dictionary dict);
@@ -151,7 +158,7 @@ static void dict_error2(Dictionary dict, const char * s, const char *s2)
 	if (s2)
 	{
 		prt_error("Error: While parsing dictionary \"%s\":\n"
-		          "%s %s\n\t Line %d, next tokens: %s\n",
+		          "%s \"%s\"\n\t Line %d, next tokens: %s\n",
 		          dict->name, s, s2, dict->line_number, tokens);
 	}
 	else
@@ -193,16 +200,6 @@ static bool get_character(Dictionary dict, int quote_mode, utf8char uc)
 		/* Skip over all comments */
 		if ((c == '%') && (!quote_mode))
 		{
-			if (0 == strncmp(dict->pin, SUPPRESS, sizeof(SUPPRESS)-1))
-			{
-				const char *nl = strchr(dict->pin + sizeof(SUPPRESS)-1, '\n');
-				if (NULL != nl)
-				{
-					dict->suppress_warning =
-						strndup(dict->pin + sizeof(SUPPRESS)-1,
-					           nl - dict->pin - sizeof(SUPPRESS) + 1);
-				}
-			}
 			while ((c != 0x0) && (c != '\n')) c = *(dict->pin++);
 			if (c == 0x0) break;
 			dict->line_number++;
@@ -236,7 +233,7 @@ static bool get_character(Dictionary dict, int quote_mode, utf8char uc)
 			uc[i] = c;
 			i++;
 		}
-		dict_error(dict, "UTF8 char is too long");
+		dict_error(dict, "UTF8 char is too long.");
 		return false;
 	}
 	uc[0] = 0x0;
@@ -299,7 +296,7 @@ static bool link_advance(Dictionary dict)
 		bool ok = get_character(dict, false, c);
 		if (!ok) return false;
 	}
-	while (lg_isspace(c[0]));
+	while (lg_isspace((unsigned char)c[0]));
 
 	quote_mode = false;
 
@@ -307,25 +304,25 @@ static bool link_advance(Dictionary dict)
 	for (;;)
 	{
 		if (i > MAX_TOKEN_LENGTH-3) {
-			dict_error(dict, "Token too long");
+			dict_error(dict, "Token too long.");
 			return false;
 		}
 		if (quote_mode) {
 			if (c[0] == '"' &&
 			    /* Check the next character too, to allow " in words */
 			    (*dict->pin == ':' || *dict->pin == ';' ||
-			    lg_isspace(*dict->pin))) {
+			    lg_isspace((unsigned char)*dict->pin))) {
 
 				dict->token[i] = '\0';
 				return true;
 			}
-			if (lg_isspace(c[0])) {
-				dict_error(dict, "White space inside of token");
+			if (lg_isspace((unsigned char)c[0])) {
+				dict_error(dict, "White space inside of token.");
 				return false;
 			}
 			if (c[0] == '\0')
 			{
-				dict_error(dict, "EOF while reading quoted token");
+				dict_error(dict, "EOF while reading quoted token.");
 				return false;
 			}
 
@@ -350,7 +347,7 @@ static bool link_advance(Dictionary dict)
 				dict->token[0] = '\0';
 				return true;
 			}
-			if (lg_isspace(c[0])) {
+			if (lg_isspace((unsigned char)c[0])) {
 				dict->token[i] = '\0';
 				return true;
 			}
@@ -397,35 +394,28 @@ static bool check_connector(Dictionary dict, const char * s)
 	}
 	if (*s == '@') s++;
 	if (('h' == *s) || ('d' == *s)) s++;
-	if (!isupper((unsigned char)*s)) {
-		dict_error(dict, "Connectors must start with uppercase after an optional h or d.");
+	if (!is_connector_name_char(*s)) {
+		dict_error2(dict, "Invalid character in connector "
+		            "(connectors must start with an uppercase letter "
+		            "after an optional \"h\" or \"d\"):", (char[]){*s, '\0'});
 		return false;
 	}
-	/* Note that IDx when x is a subscript is allowed (to allow e.g. ID4id+). */
-	if ((*s == 'I') && (*(s+1) == 'D') && isupper((unsigned char)*(s+2))) {
-		dict_error(dict, "Connectors beginning with \"ID\" are forbidden");
+	if (*s == '_')
+	{
+		dict_error(dict, "Invalid character in connector "
+		           "(an initial \"_\" is reserved for internal use).");
 		return false;
 	}
 
-	bool connector_base = true;
-	s++; /* The first uppercase has been validated above. */
-	while (*(s+1)) {
-		if ((!isalnum((unsigned char)*s)) && (*s != WILD_TYPE)) {
-			dict_error(dict, "All letters of a connector must be ASCII alpha-numeric.");
+	/* The first uppercase has been validated above. */
+	do { s++; } while (is_connector_name_char(*s));
+	while (s[1]) {
+		if (!is_connector_subscript_char(*s) && (*s != WILD_TYPE)) {
+			dict_error2(dict, "Invalid character in connector subscript "
+			            "(only lowercase letters, digits, and \"*\" are allowed):",
+			            (char[]){*s, '\0'});
 			return false;
 		}
-		if (isupper((unsigned char)*s))
-		{
-			if (!connector_base)
-			{
-				dict_error(dict, "Connector subscript contains uppercase.");
-				return false;
-			}
-		}
-		else
-		{
-			connector_base = false;
-		}
 		s++;
 	}
 	return true;
@@ -490,8 +480,8 @@ NO_SAN_DICT
 static inline int dict_order_strict(const char *s, const Dict_node * dn)
 {
 	const char * t = dn->string;
-	while (*s != '\0' && *s == *t) {s++; t++;}
-	return ((*s == SUBSCRIPT_MARK)?(1):(*s))  -  ((*t == SUBSCRIPT_MARK)?(1):(*t));
+	while ((*s == *t) && (*s != '\0')) { s++; t++; }
+	return (*s - *t);
 }
 
 /**
@@ -513,7 +503,7 @@ NO_SAN_DICT
 static inline int dict_order_bare(const char *s, const Dict_node * dn)
 {
 	const char * t = dn->string;
-	while (*s != '\0' && *s == *t) {s++; t++;}
+	while ((*s == *t) && (*s != '\0')) { s++; t++; }
 	return (*s)  -  ((*t == SUBSCRIPT_MARK)?(0):(*t));
 }
 
@@ -545,7 +535,7 @@ static inline int dict_order_wild(const char * s, const Dict_node * dn)
 	const char * t = dn->string;
 
 	lgdebug(+D_DOW, "search-word='%s' dict-word='%s'\n", s, t);
-	while((*s != '\0') && (*s != SUBSCRIPT_MARK) && (*s == *t)) {s++; t++;}
+	while((*s == *t) && (*s != SUBSCRIPT_MARK) && (*s != '\0')) { s++; t++; }
 
 	if (*s == WILD_TYPE) return 0;
 
@@ -555,6 +545,13 @@ static inline int dict_order_wild(const char * s, const Dict_node * dn)
 }
 #undef D_DOW
 
+static inline Dict_node * dict_node_new(void)
+{
+	return (Dict_node*) malloc(sizeof(Dict_node));
+}
+
+/* ======================================================================== */
+#if 0
 /**
  * dict_match --  return true if strings match, else false.
  * A "bare" string (one without a subscript) will match any corresponding
@@ -565,7 +562,7 @@ static inline int dict_order_wild(const char * s, const Dict_node * dn)
  */
 static bool dict_match(const char * s, const char * t)
 {
-	while ((*s != '\0') && (*s == *t)) { s++; t++; }
+	while ((*s == *t) && (*s != '\0')) { s++; t++; }
 
 	if (*s == *t) return true; /* both are '\0' */
 	if ((*s == 0) && (*t == SUBSCRIPT_MARK)) return true;
@@ -574,13 +571,6 @@ static bool dict_match(const char * s, const char * t)
 	return false;
 }
 
-/* ======================================================================== */
-
-static inline Dict_node * dict_node_new(void)
-{
-	return (Dict_node*) malloc(sizeof(Dict_node));
-}
-
 /**
  * prune_lookup_list -- discard all list entries that don't match string
  * Walk the lookup list (of right links), discarding all nodes that do
@@ -617,15 +607,19 @@ static Dict_node * prune_lookup_list(Dict_node * restrict llist, const char * re
 	}
 	return llist;
 }
+#endif
 
 /* ======================================================================== */
 static bool subscr_match(const char *s, const Dict_node * dn)
 {
 	const char * s_sub = strrchr(s, SUBSCRIPT_MARK);
-	const char * t_sub;
+	const char * t_sub = strrchr(dn->string, SUBSCRIPT_MARK);
 
-	if (NULL == s_sub) return true;
-	t_sub = strrchr(dn->string, SUBSCRIPT_MARK);
+	if (NULL == s_sub)
+	{
+		if (NULL == t_sub) return true;
+		return !is_idiom_word(t_sub);
+	}
 	if (NULL == t_sub) return false;
 	if (0 == strcmp(s_sub, t_sub)) return true;
 
@@ -642,10 +636,9 @@ static Dict_node *
 rdictionary_lookup(Dict_node * restrict llist,
                    Dict_node * restrict dn,
                    const char * restrict s,
-                   bool match_idiom,
+                   bool boolean_lookup,
                    int (*dict_order)(const char *, const Dict_node *))
 {
-	/* see comment in dictionary_lookup below */
 	int m;
 	Dict_node * dn_new;
 	if (dn == NULL) return llist;
@@ -654,11 +647,11 @@ rdictionary_lookup(Dict_node * restrict llist,
 
 	if (m >= 0)
 	{
-		llist = rdictionary_lookup(llist, dn->right, s, match_idiom, dict_order);
+		llist = rdictionary_lookup(llist, dn->right, s, boolean_lookup, dict_order);
 	}
-	if ((m == 0) && (match_idiom || !is_idiom_word(dn->string)) &&
-		 (dict_order != dict_order_wild || subscr_match(s, dn)))
+	if ((m == 0) && (dict_order != dict_order_wild || subscr_match(s, dn)))
 	{
+		if (boolean_lookup) return dn;
 		dn_new = dict_node_new();
 		*dn_new = *dn;
 		dn_new->right = llist;
@@ -667,7 +660,7 @@ rdictionary_lookup(Dict_node * restrict llist,
 	}
 	if (m <= 0)
 	{
-		llist = rdictionary_lookup(llist, dn->left, s, match_idiom, dict_order);
+		llist = rdictionary_lookup(llist, dn->left, s, boolean_lookup, dict_order);
 	}
 	return llist;
 }
@@ -676,8 +669,6 @@ rdictionary_lookup(Dict_node * restrict llist,
  * file_lookup_list() - return list of words in the file-backed dictionary.
  *
  * Returns a pointer to a lookup list of the words in the dictionary.
- * Matches include words that appear in idioms.  To exclude idioms, use
- * abridged_lookup_list() to obtain matches.
  *
  * This list is made up of Dict_nodes, linked by their right pointers.
  * The node, file and string fields are copied from the dictionary.
@@ -686,18 +677,12 @@ rdictionary_lookup(Dict_node * restrict llist,
  */
 Dict_node * file_lookup_list(const Dictionary dict, const char *s)
 {
-	Dict_node * llist =
-		rdictionary_lookup(NULL, dict->root, s, true, dict_order_bare);
-	llist = prune_lookup_list(llist, s);
-	return llist;
+	return rdictionary_lookup(NULL, dict->root, s, false, dict_order_bare);
 }
 
 bool file_boolean_lookup(Dictionary dict, const char *s)
 {
-	Dict_node *llist = file_lookup_list(dict, s);
-	bool boool = (llist != NULL);
-	file_free_lookup(llist);
-	return boool;
+	return !!rdictionary_lookup(NULL, dict->root, s, true, dict_order_bare);
 }
 
 void file_free_lookup(Dict_node *llist)
@@ -728,25 +713,23 @@ void free_insert_list(Dict_node *ilist)
  */
 Dict_node * file_lookup_wild(Dictionary dict, const char *s)
 {
-	bool lookup_idioms = test_enabled("lookup-idioms");
 	char * ds = strrchr(s, SUBSCRIPT_DOT); /* Only the rightmost dot is a
 	                                          candidate for SUBSCRIPT_DOT */
 	char * ws = strrchr(s, WILD_TYPE);     /* A SUBSCRIPT_DOT can only appear
                                              after a wild-card */
 	Dict_node * result;
-	char * stmp = strdup(s);
+	char * stmp = strdupa(s);
 
 	/* It is not a SUBSCRIPT_DOT if it is at the end or before the wild-card.
 	 * E.g: "Dr.", "i.*", "." */
 	if ((NULL != ds) && ('\0' != ds[1]) && ((NULL == ws) || (ds > ws)))
 		stmp[ds-s] = SUBSCRIPT_MARK;
 
-	result =
-	 rdictionary_lookup(NULL, dict->root, stmp, lookup_idioms, dict_order_wild);
-	free(stmp);
+	result = rdictionary_lookup(NULL, dict->root, stmp, false, dict_order_wild);
 	return result;
 }
 
+#if 0
 /**
  * abridged_lookup_list() - return lookup list of words in the dictionary
  *
@@ -761,17 +744,14 @@ Dict_node * file_lookup_wild(Dictionary dict, const char *s)
  */
 static Dict_node * abridged_lookup_list(const Dictionary dict, const char *s)
 {
-	Dict_node *llist;
-	llist = rdictionary_lookup(NULL, dict->root, s, false, dict_order_bare);
-	llist = prune_lookup_list(llist, s);
-	return llist;
+	return rdictionary_lookup(NULL, dict->root, s, false, dict_order_bare);
 }
+#endif
 
 /**
  * strict_lookup_list() - return exact match in the dictionary
  *
  * Returns a pointer to a lookup list of the words in the dictionary.
- * Excludes any idioms that contain the word.
  *
  * This list is made up of Dict_nodes, linked by their right pointers.
  * The node, file and string fields are copied from the dictionary.
@@ -783,10 +763,7 @@ static Dict_node * abridged_lookup_list(const Dictionary dict, const char *s)
  */
 static Dict_node * strict_lookup_list(const Dictionary dict, const char *s)
 {
-	Dict_node *llist;
-	llist = rdictionary_lookup(NULL, dict->root, s, false, dict_order_strict);
-	llist = prune_lookup_list(llist, s);
-	return llist;
+	return rdictionary_lookup(NULL, dict->root, s, false, dict_order_strict);
 }
 
 /* ======================================================================== */
@@ -969,7 +946,7 @@ static unsigned int exptag_macro_add(Dictionary dict, const char *tag)
 static Exp * make_connector(Dictionary dict)
 {
 	Exp * n;
-	Dict_node *dn, *dn_head;
+	Dict_node *dn;
 	int i;
 
 	i = strlen(dict->token) - 1;  /* this must be +, - or $ if a connector */
@@ -979,18 +956,21 @@ static Exp * make_connector(Dictionary dict)
 	{
 		/* If we are here, token is a word */
 		patch_subscript(dict->token);
-		dn_head = abridged_lookup_list(dict, dict->token);
-		dn = dn_head;
-		while ((dn != NULL) && (strcmp(dn->string, dict->token) != 0))
-		{
-			dn = dn->right;
-		}
+		dn = strict_lookup_list(dict, dict->token);
 		if (dn == NULL)
 		{
-			file_free_lookup(dn_head);
-			dict_error(dict, "Perhaps missing + or - in a connector.\n"
+			file_free_lookup(dn);
+			dict_error2(dict, "Perhaps missing + or - in a connector.\n"
 			                 "Or perhaps you forgot the subscript on a word.\n"
-			                 "Or perhaps a word is used before it is defined.");
+			                 "Or perhaps the word is used before it is defined:",
+			                 dict->token);
+			return NULL;
+		}
+		if (dn->right != NULL)
+		{
+			file_free_lookup(dn);
+			dict_error2(dict, "Referencing a duplicate word:", dict->token);
+			/* Note: A word which becomes duplicate latter evades this check. */
 			return NULL;
 		}
 
@@ -999,7 +979,7 @@ static Exp * make_connector(Dictionary dict)
 		n->tag_id = exptag_macro_add(dict, dn->string);
 		if (n->tag_id != 0) n->tag_type = Exptag_macro;
 
-		file_free_lookup(dn_head);
+		file_free_lookup(dn);
 	}
 	else
 	{
@@ -1198,7 +1178,7 @@ static Exp *make_expression(Dictionary dict)
 			}
 			else if ((strcmp(dict->token, "or") != 0) &&
 			         (strcmp(dict->token, "and") != 0) &&
-			         isalpha(dict->token[0]))
+			         isalpha((unsigned char)dict->token[0]))
 			{
 				const char *bad = valid_dialect_name(dict->token);
 				if (bad != NULL)
@@ -1304,7 +1284,7 @@ static Exp *make_expression(Dictionary dict)
 		{
 			if (e_head->type != op)
 			{
-				dict_error(dict, "\"and\" and \"or\" at the same level in an expression");
+				dict_error(dict, "\"and\" and \"or\" at the same level in an expression.");
 				return NULL;
 			}
 		}
@@ -1415,6 +1395,53 @@ static Dict_node * dsw_vine_to_tree (Dict_node *root, int size)
 }
 
 /* ======================================================================== */
+/**
+ * Notify about a duplicate word, unless allowed or it is an idiom definition.
+ * Idioms are exempt because historically they couldn't be
+ * differentiated using a subscript if duplicate definitions were
+ * convenience (and also they were not inserted into the dictionary so
+ * their duplicate check got neglected).
+ *
+ * The following dictionary definition allows duplicate words:
+ * #define allow-duplicate-words true
+ * An idiom duplicate check can be requested using the test "dup-idioms".
+ */
+static bool dup_word_error(Dictionary dict, Dict_node *newnode)
+{
+	static int dup_idioms = -1;
+	static int allow_duplicate_words = -1;
+
+	if (allow_duplicate_words == 1) return false;
+	if (allow_duplicate_words == -1)
+	{
+		const char *s = linkgrammar_get_dict_define(dict, "allow-duplicate-words");
+
+		allow_duplicate_words = ((s != NULL) && (0 == strcasecmp(s, "true")));
+		if (allow_duplicate_words) return false;
+		if (dup_idioms == -1) dup_idioms = !!test_enabled("dup-idioms");
+	}
+
+	/* FIXME: Make central. */
+	static Exp null_exp =
+	{
+		.type = AND_type,
+		.operand_first = NULL,
+		.operand_next = NULL,
+	};
+
+	if (!contains_underbar(newnode->string) || dup_idioms)
+	{
+		dict_error2(dict, "Ignoring word which has been multiply defined:",
+		            newnode->string);
+		/* Too late to skip insertion - insert it with a null expression. */
+		newnode->exp = &null_exp;
+
+		return true;
+	}
+
+	return false;
+}
+
 /**
  * Insert the new node into the dictionary below node n.
  * "newnode" left and right fields are NULL, and its string is already
@@ -1429,27 +1456,10 @@ Dict_node *insert_dict(Dictionary dict, Dict_node *n, Dict_node *newnode)
 {
 	if (NULL == n) return newnode;
 
-	static Exp null_exp =
-	{
-		.type = AND_type,
-		.operand_first = NULL,
-		.operand_next = NULL,
-	};
 	int comp = dict_order_strict(newnode->string, n);
 
-	if (0 == comp &&
-	    /* Suppress reporting duplicate idioms until they are fixed. */
-	    (!contains_underbar(newnode->string) || test_enabled("dup-idioms")))
-	{
-		char t[80+MAX_TOKEN_LENGTH];
-		snprintf(t, sizeof(t),
-		         "Ignoring word \"%s\", which has been multiply defined:",
-		         newnode->string);
-		dict_error(dict, t);
-		/* Too late to skip insertion - insert it with a null expression. */
-		newnode->exp = &null_exp;
-		comp = -1;
-	}
+	if ((0 == comp) && dup_word_error(dict, newnode))
+	    comp = -1;
 
 	if (comp < 0)
 	{
@@ -1474,17 +1484,6 @@ Dict_node *insert_dict(Dictionary dict, Dict_node *n, Dict_node *newnode)
 	/* return rebalance(n); Uncomment to get an AVL tree */
 }
 
-/**
- * Find if a warning symbol exists in the currently suppress list.
- * The warning symbols are constructed in a way that disallow overlap
- * matching.
- */
-static bool is_warning_suppressed(Dictionary dict, const char *warning_symbol)
-{
-	if (NULL == dict->suppress_warning) return false;
-	return (NULL != strstr(dict->suppress_warning, warning_symbol));
-}
-
 /**
  * Remember the length_limit definitions in a list according to their order.
  * The order is kept to allow later more specific definitions to override
@@ -1554,7 +1553,7 @@ static void insert_length_limit(Dictionary dict, Dict_node *dn)
  */
 void insert_list(Dictionary dict, Dict_node * p, int l)
 {
-	Dict_node * dn, *dn_head, *dn_second_half;
+	Dict_node * dn, *dn_second_half;
 	int k, i; /* length of first half */
 
 	if (l == 0) return;
@@ -1570,10 +1569,11 @@ void insert_list(Dictionary dict, Dict_node * p, int l)
 	dn_second_half = dn->left;
 	dn->left = dn->right = NULL;
 
-	if (is_idiom_word(dn->string))
+	const char *sm = strchr(dn->string, SUBSCRIPT_MARK);
+	if ((NULL != sm) && ('_' == sm[1]))
 	{
 		prt_error("Warning: Word \"%s\" found near line %d of \"%s\".\n"
-		        "\tWords ending \".I\" are reserved for idioms.\n"
+		        "\tWords ending \"._\" are reserved for internal use.\n"
 		        "\tThis word will be ignored.\n",
 		        dn->string, dict->line_number, dict->name);
 		free(dn);
@@ -1588,44 +1588,6 @@ void insert_list(Dictionary dict, Dict_node * p, int l)
 		dict->root = insert_dict(dict, dict->root, dn);
 		insert_length_limit(dict, dn);
 		dict->num_entries++;
-
-		if ((verbosity_level(D_DICT+0) && !is_warning_suppressed(dict, DUP_BASE)) ||
-		    verbosity_level(D_SPEC+3))
-		{
-			/* Warn if there are words with a subscript that match a bare word. */
-			const char *sm = strchr(dn->string, SUBSCRIPT_MARK);
-			char *bareword;
-
-			if (NULL != sm)
-			{
-				bareword = strdupa(dn->string);
-				bareword[sm - dn->string] = '\0';
-			}
-			else
-			{
-				bareword = (char *)dn->string;
-			}
-
-			if ((dn_head = strict_lookup_list(dict, bareword)) != NULL)
-			{
-				bool match_found = false;
-				for (Dict_node *dnx = dn_head; dnx != NULL; dnx = dnx->right) {
-					if (0 != strcmp(dnx->string, dn->string))
-					{
-						if (!match_found)
-						{
-							prt_error("Warning: The word \"%s\" found near line "
-										 "%d of \"%s\"\n\t matches the following words:",
-										 dn->string, dict->line_number, dict->name);
-							match_found = true;
-						}
-						prt_error("\t%s", dnx->string);
-					}
-				}
-				if (match_found) prt_error("\n");
-				file_free_lookup(dn_head);
-			}
-		}
 	}
 
 	insert_list(dict, p, k);
@@ -1890,12 +1852,6 @@ static bool read_entry(Dictionary dict)
 
 	dict->insert_entry(dict, dn, i);
 
-	if (dict->suppress_warning)
-	{
-		free((void *)dict->suppress_warning);
-		dict->suppress_warning = NULL;
-	}
-
 	/* pass the ; */
 	if (!link_advance(dict))
 	{
@@ -1925,7 +1881,7 @@ static void rprint_dictionary_data(Dict_node * n)
 {
 	if (n == NULL) return;
 	rprint_dictionary_data(n->left);
-	printf("%s: %s\n", n->string, lg_exp_stringify(n->exp));
+	printf("%s: %s\n", n->string, exp_stringify(n->exp));
 	rprint_dictionary_data(n->right);
 }
 
