@@ -1,11 +1,10 @@
-
 /*
  * read-sql.h
  *
  * Read in dictionary from an SQL DB.
  * Keeping it simple for just right now, and using SQLite.
  *
- * The goal of reading the dictioary from SQL is to enable some 
+ * The goal of reading the dictionary from SQL is to enable some
  * other process (machine-learning algo) to dynamically update
  * the dictionary.
  *
@@ -17,13 +16,8 @@
 
 #include "link-includes.h"
 
-#ifdef HAVE_SQLITE
-bool check_db(const char *lang);
+#ifdef HAVE_SQLITE3
 Dictionary dictionary_create_from_db(const char *lang);
-#else
-
-static inline bool check_db(const char *lang) { return false; }
-static inline Dictionary dictionary_create_from_db(const char *lang) { return NULL; }
-#endif /* HAVE_SQLITE */
+#endif /* HAVE_SQLITE3 */
 
 #endif /* READ_SQL_H */
