@@ -37,7 +37,7 @@ struct Table_connector_s
 {
 	Table_connector  *next;
 	int              l_id, r_id;
-	count_t          count;
+	Count_bin          count;
 	unsigned int     null_count;
 	size_t           hash;
 };
@@ -113,12 +113,6 @@ struct count_context_s
 #define MAX_TABLE_SIZE(s) (s / 10) /* Low load factor, for speed */
 #define MAX_LOG2_TABLE_SIZE ((sizeof(size_t)==4) ? 25 : 34)
 
-static void free_table(count_context_t *ctxt)
-{
-	ctxt->table = NULL;
-	ctxt->table_size = 0;
-}
-
 #if HAVE_THREADS_H
 /* Each thread will get it's own version of the `kept_table`.
  * If the program creates zillions of threads, then there will
@@ -205,9 +199,9 @@ static void table_alloc(count_context_t *ctxt, unsigned int shift)
  * sent->length as a hint for the initial table size. Usually, this
  * saves on dynamic table growth, which is costly.
  * */
-static void init_table(count_context_t *ctxt, Sentence sent)
+static void init_table(count_context_t *ctxt)
 {
-	if (ctxt->table) free_table(ctxt);
+	Sentence sent = ctxt->sent;
 
 	/* A piecewise exponential function determines the size of the
 	 * hash table. Probably should make use of the actual number of
@@ -233,12 +227,17 @@ static void init_table(count_context_t *ctxt, Sentence sent)
 
 static void free_table_lrcnt(count_context_t *ctxt)
 {
+	if (ctxt->is_short) return;
+
 	if (verbosity_level(D_COUNT))
 	{
 		unsigned int nonzero = 0, any_null = 0, zero = 0, non_max_null = 0;
 		Pool_location loc = { 0 };
 		Table_lrcnt *t;
 
+		/* Note: Due to a current pool_next() problem for vector elements,
+		 * the stats results here may be slightly skewed.  See the comment
+		 * on that in memory-pool.h. */
 		while ((t = pool_next(ctxt->sent->wordvec_pool, &loc)) != NULL)
 		{
 			if (t->status == -1) continue;
@@ -281,8 +280,11 @@ static void free_table_lrcnt(count_context_t *ctxt)
 	}
 }
 
-static void init_table_lrcnt(count_context_t *ctxt, Sentence sent)
+static void init_table_lrcnt(count_context_t *ctxt)
 {
+	if (ctxt->is_short) return;
+	Sentence sent = ctxt->sent;
+
 	for (unsigned int dir = 0; dir < 2; dir++)
 	{
 		const size_t sz = sizeof(wordvecp) * ctxt->table_lrcnt_size[dir];
@@ -329,7 +331,6 @@ static void table_stat(count_context_t *ctxt)
 	size_t N = 0;          /* NULL table slots */
 	size_t null_count[256] = { 0 };   /* null_count histogram */
 	int chain_length[64] = { 0 };     /* Chain length histogram */
-	int n;                 /* For printf() pretty printing */
 	bool table_stat_entries = test_enabled("count-table-entries");
 
 	for (size_t i = 0; i < ctxt->table_size; i++)
@@ -345,7 +346,7 @@ static void table_stat(count_context_t *ctxt)
 		{
 			assert(t->hash != 0, "Invalid hash value: 0");
 			assert((hist_total(&t->count)>=0)&&(hist_total(&t->count) <= INT_MAX),
-			       "Invalid count %lld", hist_total(&t->count));
+			       "Invalid count %"COUNT_FMT, hist_total(&t->count));
 			assert(t->l_id < (int)ctxt->sent->length ||
 			       ((t->l_id >= 255)&&(t->l_id < (int)ctxt->table_lrcnt_size[0])),
 			       "invalid l_id %d", t->l_id);
@@ -391,7 +392,7 @@ static void table_stat(count_context_t *ctxt)
 		for (unsigned int nc = 0; nc < ARRAY_SIZE(null_count); nc++)
 		{
 			if (0 != null_count[nc])
-				printf("%u: %d\n", nc, null_count[nc]);
+				printf("%u: %zu\n", nc, null_count[nc]);
 		}
 	}
 
@@ -408,8 +409,8 @@ static void table_stat(count_context_t *ctxt)
 				{
 					if (t->null_count != nc) continue;
 
-					printf("[%u]%n", i, &n);
-					printf("%*d %5d c=%lld\n",  15-n, t->l_id, t->r_id, t->count);
+					int n = printf("[%zu]", i);
+					printf("%*d %5d c=%"COUNT_FMT"\n",  15-n, t->l_id, t->r_id, t->count);
 				}
 			}
 		}
@@ -441,11 +442,11 @@ static void table_grow(count_context_t *ctxt)
 /**
  * Stores the value in the table.  Assumes it's not already there.
  */
-static count_t table_store(count_context_t *ctxt,
+static Count_bin table_store(count_context_t *ctxt,
                            int lw, int rw,
                            const Connector *le, const Connector *re,
                            unsigned int null_count,
-                           size_t hash, count_t c)
+                           size_t hash, Count_bin c)
 {
 	if (ctxt->table_available_count == 0) table_grow(ctxt);
 
@@ -475,7 +476,7 @@ static count_t table_store(count_context_t *ctxt,
  * the entry is not found).
  * @return The count for this quintuple if there, NULL otherwise.
  */
-inline count_t *
+inline Count_bin *
 table_lookup(count_context_t *ctxt, int lw, int rw,
              const Connector *le, const Connector *re,
              unsigned int null_count, size_t *hash)
@@ -502,6 +503,11 @@ table_lookup(count_context_t *ctxt, int lw, int rw,
 	return NULL;
 }
 
+extern  Count_bin *
+table_lookup(count_context_t *, int, int,
+             const Connector *, const Connector *,
+             unsigned int, size_t *);
+
 /**
  * Generate a word skip vector.
  * This implements an idea similar to the Boyer-Moore algo for searches.
@@ -594,7 +600,7 @@ static void generate_word_skip_vector(count_context_t *ctxt, wordvecp wv,
  * skipped because the cache indicates they yield a zero count.)
  * @return Cache entry for the given range. Possible values:
  *    NULL - A nonzero count may be encountered for \c null_count>=lnull_start.
- *    Table_lrcnt_zero - A zero count would result.
+ *    lrcnt_cache_zero - A zero count would result.
  *    Cache pointer - An update for \c null_count>=lnull_start is needed.
  */
 static Table_lrcnt *is_lrcnt(count_context_t *ctxt, int dir, Connector *c,
@@ -695,12 +701,12 @@ static bool is_panic(count_context_t *ctxt)
 }
 
 #define NO_COUNT -1
-#ifdef PERFORM_COUNT_HISTOGRAMMING
-#define INIT_NO_COUNT (count_t){.total = NO_COUNT}
+#if PERFORM_COUNT_HISTOGRAMMING
+#define INIT_NO_COUNT (Count_bin){.total = NO_COUNT}
 #else
 #define INIT_NO_COUNT NO_COUNT
 #endif
-count_t count_unknown = INIT_NO_COUNT;
+Count_bin count_unknown = INIT_NO_COUNT;
 
 /**
  * psuedocount is used to check to see if a parse is even possible,
@@ -714,11 +720,19 @@ count_t count_unknown = INIT_NO_COUNT;
  * worst case: that the count might be non-zero.  To indicate that case,
  * return the special sentinel value \c count_unknown.
  */
-static count_t pseudocount(count_context_t * ctxt,
+static Count_bin pseudocount(count_context_t * ctxt,
                            int lw, int rw, Connector *le, Connector *re,
                            unsigned int null_count)
 {
-	count_t *count = table_lookup(ctxt, lw, rw, le, re, null_count, NULL);
+	/* This check is not necessary for correctness, but it saves CPU time.
+	 * If a cross link would result, immediatly return 0. Note that there is
+	 * no need to check here if the nearest_word fields are in the range
+	 * [lw, rw] due to the way start_word/end_word are computed, and due to
+	 * nearest_word checks in form_match_list(). */
+	if ((le != NULL) && (re != NULL) && (le->nearest_word > re->nearest_word))
+		return hist_zero();
+
+	Count_bin *count = table_lookup(ctxt, lw, rw, le, re, null_count, NULL);
 	if (NULL == count) return count_unknown;
 	return *count;
 }
@@ -728,12 +742,12 @@ static count_t pseudocount(count_context_t * ctxt,
  *  The returned value is normally unused by the callers
  *  (to be used when debugging overflows).
  */
-static bool parse_count_clamp(w_count_t *total)
+static bool parse_count_clamp(w_Count_bin *total)
 {
 	if (INT_MAX < hist_total(total))
 	{
 		/*  Sigh. Overflows can and do occur, esp for the ANY language. */
-#ifdef PERFORM_COUNT_HISTOGRAMMING
+#if PERFORM_COUNT_HISTOGRAMMING
 		total->total = INT_MAX;
 #else
 		*total = INT_MAX;
@@ -770,12 +784,12 @@ static int num_optional_words(count_context_t *ctxt, int w1, int w2)
 	 prt_error("%-*s", LBLSZ, STRINGIFY(l)) : 0, do_count)
 #define V(c) (!c?"(nil)":connector_string(c))
 #define ID(c,w) (!c?w:c->tracon_id)
-static count_t do_count1(int lineno, count_context_t *ctxt,
+static Count_bin do_count1(int lineno, count_context_t *ctxt,
                          int lw, int rw,
                          Connector *le, Connector *re,
                          unsigned int null_count);
 
-static count_t do_count(int lineno, count_context_t *ctxt,
+static Count_bin do_count(int lineno, count_context_t *ctxt,
                         int lw, int rw,
                         Connector *le, Connector *re,
                         unsigned int null_count)
@@ -785,7 +799,7 @@ static count_t do_count(int lineno, count_context_t *ctxt,
 	if (!verbosity_level(D_COUNT_TRACE))
 		return do_count1(lineno, ctxt, lw, rw, le, re, null_count);
 
-	count_t *c = table_lookup(ctxt, lw, rw, le, re, null_count, NULL);
+	Count_bin *c = table_lookup(ctxt, lw, rw, le, re, null_count, NULL);
 	char m_result[64] = "";
 	if (c != NULL)
 		snprintf(m_result, sizeof(m_result), "(M=%"COUNT_FMT")", hist_total(c));
@@ -793,7 +807,7 @@ static count_t do_count(int lineno, count_context_t *ctxt,
 	level++;
 	prt_error("%*sdo_count%s:%d lw=%d rw=%d le=%s(%d) re=%s(%d) null_count=%u\n\\",
 		level*2, "", m_result, lineno, lw, rw, V(le),ID(le,lw), V(re),ID(re,rw), null_count);
-	count_t r = do_count1(lineno, ctxt, lw, rw, le, re, null_count);
+	Count_bin r = do_count1(lineno, ctxt, lw, rw, le, re, null_count);
 	prt_error("%*sreturn%.*s:%d=%"COUNT_FMT"\n",
 	          LBLSZ+level*2, "", (!!c)*3, "(M)", lineno, hist_total(&r));
 	level--;
@@ -805,7 +819,7 @@ static count_t do_count(int lineno, count_context_t *ctxt,
 #else
 #define TRACE_LABEL(l, do_count) (do_count)
 #endif /* DO_COUNT TRACE */
-static count_t do_count(
+static Count_bin do_count(
 #ifdef DO_COUNT_TRACE
 #undef do_count
 #define do_count(...) do_count(__LINE__, __VA_ARGS__)
@@ -816,9 +830,14 @@ static count_t do_count(
                           Connector *le, Connector *re,
                           unsigned int null_count)
 {
-	w_count_t total = hist_zero();;
+	w_Count_bin total = hist_zero();
 	int start_word, end_word, w;
 
+	/* This check is not necessary for correctness. It typically saves
+	 * significant CPU and Table_connector memory because in many cases a
+	 * linkage is not possible due to nearest_word restrictions. */
+	if (!valid_nearest_words(le, re, lw, rw)) return hist_zero();
+
 	if (is_panic(ctxt)) return hist_zero();
 
 	/* TODO: static_assert() that null_count is an unsigned int. */
@@ -826,7 +845,7 @@ static count_t do_count(
 
 	size_t h = 0; /* Initialized value only needed to prevent a warning. */
 	{
-		count_t* const c = table_lookup(ctxt, lw, rw, le, re, null_count, &h);
+		Count_bin* const c = table_lookup(ctxt, lw, rw, le, re, null_count, &h);
 		if (c != NULL) return *c;
 		/* The table entry is to be updated with the found linkage count
 		 * before we return. The hash h will be used to locate it. */
@@ -1082,16 +1101,16 @@ static count_t do_count(
 				bool leftpcount = false;
 				bool rightpcount = false;
 
-				count_t l_any = INIT_NO_COUNT;
-				count_t r_any = INIT_NO_COUNT;
-				count_t l_cmulti = INIT_NO_COUNT;
-				count_t l_dmulti = INIT_NO_COUNT;
-				count_t l_dcmulti = INIT_NO_COUNT;
-				count_t l_bnr = INIT_NO_COUNT;
-				count_t r_cmulti = INIT_NO_COUNT;
-				count_t r_dmulti = INIT_NO_COUNT;
-				count_t r_dcmulti = INIT_NO_COUNT;
-				count_t r_bnl = (le == NULL) ? INIT_NO_COUNT : hist_zero();
+				Count_bin l_any = INIT_NO_COUNT;
+				Count_bin r_any = INIT_NO_COUNT;
+				Count_bin l_cmulti = INIT_NO_COUNT;
+				Count_bin l_dmulti = INIT_NO_COUNT;
+				Count_bin l_dcmulti = INIT_NO_COUNT;
+				Count_bin l_bnr = INIT_NO_COUNT;
+				Count_bin r_cmulti = INIT_NO_COUNT;
+				Count_bin r_dmulti = INIT_NO_COUNT;
+				Count_bin r_dcmulti = INIT_NO_COUNT;
+				Count_bin r_bnl = (le == NULL) ? INIT_NO_COUNT : hist_zero();
 
 				/* Now, we determine if (based on table only) we can see that
 				   the current range is not parsable. */
@@ -1176,7 +1195,7 @@ static count_t do_count(
 
 #define CACHE_COUNT(c, how_to_count, do_count) \
 	{ \
-		w_count_t count = (hist_total(&c) == NO_COUNT) ? \
+		w_Count_bin count = (hist_total(&c) == NO_COUNT) ? \
 			TRACE_LABEL(c, do_count) : c; \
 		how_to_count; \
 	}
@@ -1184,8 +1203,8 @@ static count_t do_count(
 				 * in the count multiplication is zero,
 				 * we know that the true total is zero. So we don't
 				 * bother counting the other term at all, in that case. */
-				w_count_t leftcount = hist_zero();
-				w_count_t rightcount = hist_zero();
+				w_Count_bin leftcount = hist_zero();
+				w_Count_bin rightcount = hist_zero();
 				if (leftpcount &&
 				    (!lrcnt_optimize || rightpcount || (0 != hist_total(&l_bnr))))
 				{
@@ -1299,15 +1318,13 @@ static count_t do_count(
 int do_parse(Sentence sent, fast_matcher_t *mchxt, count_context_t *ctxt,
              Parse_Options opts)
 {
-	count_t hist;
+	Count_bin hist;
 
 	ctxt->current_resources = opts->resources;
 	ctxt->exhausted = false;
 	ctxt->checktimer = 0;
 	ctxt->islands_ok = opts->islands_ok;
 	ctxt->mchxt = mchxt;
-	ctxt->is_short =
-		(sent->length <= min_len_word_vector) && !IS_GENERATION(ctxt->sent->dict);
 
 	hist = do_count(ctxt, -1, sent->length, NULL, NULL, sent->null_count+1);
 
@@ -1318,16 +1335,18 @@ int do_parse(Sentence sent, fast_matcher_t *mchxt, count_context_t *ctxt,
 /* sent_length is used only as a hint for the hash table size ... */
 count_context_t * alloc_count_context(Sentence sent, Tracon_sharing *ts)
 {
-	count_context_t *ctxt = (count_context_t *) xalloc (sizeof(count_context_t));
+	count_context_t *ctxt = malloc (sizeof(count_context_t));
 	memset(ctxt, 0, sizeof(count_context_t));
 
 	ctxt->sent = sent;
+	ctxt->is_short =
+		(sent->length <= min_len_word_vector) && !IS_GENERATION(ctxt->sent->dict);
 
 	/* next_id keeps the last tracon_id used, so we need +1 for array size.  */
 	for (unsigned int dir = 0; dir < 2; dir++)
 		ctxt->table_lrcnt_size[dir] = ts->next_id[!dir] + 1;
 
-	init_table_lrcnt(ctxt, sent);
+	init_table_lrcnt(ctxt);
 
 	/* consecutive blocks of this many words are considered as
 	 * one null link. */
@@ -1345,7 +1364,7 @@ count_context_t * alloc_count_context(Sentence sent, Tracon_sharing *ts)
 			         /*zero_out*/false, /*align*/false, /*exact*/false);
 	}
 
-	init_table(ctxt, sent);
+	init_table(ctxt);
 	return ctxt;
 }
 
@@ -1357,7 +1376,6 @@ void free_count_context(count_context_t *ctxt, Sentence sent)
 	           "disjunct %"PRIu64", null_count %"PRIu64"\n",
 	            ctxt->count_cost[0], ctxt->count_cost[1], ctxt->count_cost[2]);)
 
-	free_table(ctxt);
 	free_table_lrcnt(ctxt);
-	xfree(ctxt, sizeof(count_context_t));
+	free(ctxt);
 }
