@@ -352,7 +352,9 @@ Parse_set * mk_parse_set(fast_matcher_t *mchxt,
 {
 	int start_word, end_word, w;
 	Pset_bucket *xt;
-	count_t *count;
+	Count_bin *count;
+
+	if (!valid_nearest_words(le, re, lw, rw)) return NULL;
 
 	assert(null_count < 0x7fff, "Called with null_count < 0.");
 
@@ -857,5 +859,4 @@ void mark_used_disjuncts(extractor_t *pex, bool *disjunct_used)
 		for (Pset_bucket *t = pex->x_table[i]; t != NULL; t = t->next)
 			mark_used_disjunct(&t->set, disjunct_used);
 	}
-
 }
