@@ -13,23 +13,48 @@
 
 #include "api-structures.h"              // for Sentence_s
 #include "connectors.h"
-#include "dict-common/dict-api.h"        // expression_stringify
+#include "dict-common/dict-structures.h" // expression_stringify
 #include "dict-common/dict-utils.h"      // size_of_expression
 #include "print/print-util.h"            // dyn_str functions
 #include "string-set.h"
 #include "tokenize/word-structures.h"    // for Word_struct
 #include "exprune.h"
 
+/**
+ * Here is expression pruning.  This is done even before the expressions
+ * are turned into lists of disjuncts. The idea is to make the work of the
+ * next steps (building the disjuncts, removing duplicate disjuncts,
+ * disjunct pruning) lighter by first removing irrelevant connectors.
+ *
+ * The algorithm prunes connectors that can be eliminated by simple checks.
+ *
+ * A series of passes are made through the sentence, alternating
+ * left-to-right and right-to-left.  Consider the left-to-right pass (the
+ * other is symmetric).  A set S of connectors is maintained (initialized
+ * to be empty).  Now the expressions of the current word are processed.
+ * If a given left pointing connector has no connector in S to which it
+ * can be matched, then that connector is deleted. The expression is then
+ * simplified by deleting AND sub-expression with a deleted component
+ * (which may be a connector or a sub-expression), and removing deleted
+ * components from OR sub-expressions (deleting the OR sub-expression upon
+ * deleting of its last component).  Now the set S is augmented by the
+ * right connectors of the remaining disjuncts of that word.  This
+ * completes one word.  The process continues through the words from left
+ * to right.
+ * Alternate passes are made until no connector is deleted.
+ *
+ * FIXME: Mark shallow connectors on dictionary read and enhance the
+ * pruning accordingly.
+ */
+
 #define D_EXPRUNE 9
 
 #ifdef DEBUG
 #define DBG(p, w, X) \
 	if (verbosity_level(+D_EXPRUNE))\
 	{\
-		char *e = expression_stringify(x->exp);\
 		err_msg(lg_Trace, "pass%d w%zu: ", p, w);\
-		err_msg(lg_Trace, X ": %s\n", e);\
-		free(e);\
+		err_msg(lg_Trace, X ": %s\n", exp_stringify(x->exp));\
 	}
 #else /* !DEBUG */
 #define DBG(p, w, X)
@@ -124,9 +149,11 @@ static void free_connector_table(exprune_context *ctxt)
 }
 
 /**
- * This hash function only looks at the leading upper case letters of
- * the connector string, and the label fields.  This ensures that if two
- * strings match (formally), then they must hash to the same place.
+ * Use the uniquely-assigned number per uppercase part of each
+ * connector string as a hash table key.
+ *
+ * This ensures that connectors with identical uppercase parts
+ * will hash to the same place.
  */
 static inline unsigned int hash_S(condesc_t * c)
 {
@@ -142,14 +169,7 @@ static inline bool matches_S(connector_table **ct, int w, condesc_t * c)
 
 	for (e = ct[hash_S(c)]; e != NULL; e = e->next)
 	{
-		if (e->farthest_word <= 0)
-		{
-			if (w < -e->farthest_word) continue;
-		}
-		else
-		{
-			if (w > e->farthest_word) continue;
-		}
+		if (w > e->farthest_word) continue;
 		if (easy_match_desc(e->condesc, c)) return true;
 	}
 	return false;
@@ -157,18 +177,12 @@ static inline bool matches_S(connector_table **ct, int w, condesc_t * c)
 
 /* ================================================================= */
 /**
- * Here is expression pruning.  This is done even before the expressions
- * are turned into lists of disjuncts.
- *
- * This uses many of the same data structures and functions that are used
- * by prune.
- *
  * The purge operations remove all irrelevant stuff from the expression,
  * and free the purged stuff.  A connector is deemed irrelevant if it
  * doesn't match anything in the set S.
  *
  * If an OR or AND type expression node has one child, we can replace it
- * by its child.  This, of course, is not really necessary, except for
+ * by it's child.  This, of course, is not really necessary, except for
  * performance.
  */
 
@@ -177,33 +191,85 @@ static Exp* purge_Exp(exprune_context *ctxt, int, Exp *, char);
 /**
  * Get rid of the current_elements with null expressions
  */
-static E_list * or_purge_E_list(exprune_context *ctxt, int w, E_list *l, char dir)
+static bool or_purge_operands(exprune_context *ctxt, int w, Exp *e, char dir)
 {
-	E_list * el;
-	if (l == NULL) return NULL;
-	if ((l->e = purge_Exp(ctxt, w, l->e, dir)) == NULL)
+#if NOTYET
+	const double nullexp_nonexistence = -9999;
+	double nullexp_mincost = nullexp_nonexistence;
+	int nullexp_count = 0;
+#endif
+
+	for (Exp **opdp = &e->operand_first; *opdp != NULL; /* See: NEXT */)
 	{
-		el = or_purge_E_list(ctxt, w, l->next, dir);
-		return el;
+		Exp *opd = *opdp;
+
+#if NOTYET
+		if ((opd->type == AND_type) && (opd->operand_first == NULL))
+		{
+			if (opd->cost > nullexp_mincost) nullexp_mincost = opd->cost;
+			nullexp_count++;
+		}
+		else
+#endif
+		if (purge_Exp(ctxt, w, opd, dir) == NULL)
+		{
+			*opdp = opd->operand_next; /* Discard element + NEXT */
+			continue;
+		}
+
+		opdp = &opd->operand_next; /* NEXT */
+	};
+
+#if NOTYET
+	if ((nullexp_count > 1) && (nullexp_mincost != nullexp_nonexistence))
+	{
+		bool nullexp_retained = false;
+
+		for (Exp **opdp = &e->operand_first; *opdp != NULL; /* See: NEXT */)
+		{
+			Exp *opd = *opdp;
+
+			if ((opd->type == AND_type) && (opd->operand_first == NULL))
+			{
+				if (!nullexp_retained && opd->cost == nullexp_mincost)
+				{
+					 nullexp_retained = true;
+				}
+				else
+				{
+					*opdp = opd->operand_next; /* Discard element + NEXT */
+					continue;
+				}
+			}
+
+			opdp = &opd->operand_next; /* NEXT */
+		}
 	}
-	l->next = or_purge_E_list(ctxt, w, l->next, dir);
-	return l;
+#endif
+
+	return (e->operand_first != NULL);
 }
 
 /**
  * Returns true iff the length of the disjunct list is 0.
  * If this is the case, it frees the structure rooted at l.
  */
-static bool and_purge_E_list(exprune_context *ctxt, int w, E_list *l, char dir)
+static bool and_purge_operands(exprune_context *ctxt, int w, Exp *e, char dir)
 {
-	if (l == NULL) return true;
-	if ((l->e = purge_Exp(ctxt, w, l->e, dir)) == NULL)
+	for (Exp **opdp = &e->operand_first; *opdp != NULL; /* See: NEXT */)
 	{
-		return false;
-	}
-	if (!and_purge_E_list(ctxt, w, l->next, dir))
-	{
-		return false;
+		Exp *opd = *opdp;
+
+#ifdef NOTYET
+		if ((opd->type == AND_type) && (opd->operand_first == NULL))
+		{
+			e->cost += opd->cost;
+			*opdp = opd->operand_next; /* Discard element + NEXT */
+		}
+#endif
+
+		if (purge_Exp(ctxt, w, opd, dir) == NULL) return false;
+		opdp = &opd->operand_next; /* NEXT */
 	}
 	return true;
 }
@@ -219,7 +285,7 @@ static Exp* purge_Exp(exprune_context *ctxt, int w, Exp *e, char dir)
 	{
 		if (e->dir == dir)
 		{
-			if (!matches_S(ctxt->ct, w, e->u.condesc))
+			if (!matches_S(ctxt->ct, (dir == '-') ? w : -w, e->condesc))
 			{
 				ctxt->N_deleted++;
 				return NULL;
@@ -231,26 +297,20 @@ static Exp* purge_Exp(exprune_context *ctxt, int w, Exp *e, char dir)
 
 	if (e->type == AND_type)
 	{
-		if (!and_purge_E_list(ctxt, w, e->u.l, dir))
-		{
-			return NULL;
-		}
+		if (!and_purge_operands(ctxt, w, e, dir)) return NULL;
 	}
-	else /* if we are here, its OR_type */
+	else /* if we are here, it's OR_type */
 	{
-		e->u.l = or_purge_E_list(ctxt, w, e->u.l, dir);
-		if (e->u.l == NULL)
-		{
-			return NULL;
-		}
+		if (!or_purge_operands(ctxt, w, e, dir)) return NULL;
 	}
 
 	/* Unary node elimination (for a slight performance improvement). */
-	if ((e->u.l != NULL) && (e->u.l->next == NULL))
+	if ((e->operand_first != NULL) && (e->operand_first->operand_next == NULL))
 	{
-		Exp *ne = e->u.l->e;
-		ne->cost += e->cost;
-		return ne;
+		Exp *opd = e->operand_first;
+		opd->cost += e->cost;
+		opd->operand_next = e->operand_next;
+		*e = *opd;
 	}
 
 	return e;
@@ -267,7 +327,8 @@ static void zero_connector_table(exprune_context *ctxt)
  * This function puts connector c into the connector table
  * if one like it isn't already there.
  */
-static void insert_connector(exprune_context *ctxt, int farthest_word, condesc_t * c)
+static void insert_connector(exprune_context *ctxt, int farthest_word,
+                             condesc_t *c)
 {
 	unsigned int h;
 	connector_table *e;
@@ -278,9 +339,7 @@ static void insert_connector(exprune_context *ctxt, int farthest_word, condesc_t
 	{
 		if (c == e->condesc)
 		{
-			{
-				if (e->farthest_word < farthest_word) e->farthest_word = farthest_word;
-			}
+			if (e->farthest_word < farthest_word) e->farthest_word = farthest_word;
 			return;
 		}
 	}
@@ -302,21 +361,16 @@ static void insert_connectors(exprune_context *ctxt, int w, Exp * e, int dir)
 	{
 		if (e->dir == dir)
 		{
-			assert(NULL != e->u.condesc, "NULL connector");
-			Connector c = { .desc = e->u.condesc };
-
-			set_connector_length_limit(&c, ctxt->opts);
-			int farthest_word = (dir == '-') ? -MAX(0, w-c.length_limit) :
-				                              w+c.length_limit;
-			insert_connector(ctxt, farthest_word, e->u.condesc);
+			assert(NULL != e->condesc, "NULL connector");
+			int farthest_word = (dir == '-') ? -e->farthest_word : e->farthest_word;
+			insert_connector(ctxt, farthest_word, e->condesc);
 		}
 	}
 	else
 	{
-		E_list *l;
-		for (l=e->u.l; l!=NULL; l=l->next)
+		for (Exp *opd = e->operand_first; opd != NULL; opd = opd->operand_next)
 		{
-			insert_connectors(ctxt, w, l->e, dir);
+			insert_connectors(ctxt, w, opd, dir);
 		}
 	}
 }
@@ -366,6 +420,7 @@ void expression_prune(Sentence sent, Parse_Options opts)
 				X_node *x = *xp;
 
 				DBG(pass, w, "l->r pass before purging");
+				//if (pass == 0 && w == 0) {printf("Exp: ");prt_exp_mem(x->exp, 0);}
 				x->exp = purge_Exp(&ctxt, w, x->exp, '-');
 				DBG(pass, w, "l->r pass after purging");
 
@@ -576,11 +631,11 @@ static int hash_disjunct(disjunct_dup_table *dt, Disjunct * d)
 	int i;
 	Connector *e;
 	i = 0;
-	for (e = d->left ; e != NULL; e = e->next)
+	for (e = d->left; e != NULL; e = e->next)
 	{
 		i = pconnector_hash(dt, e, i);
 	}
-	for (e = d->right ; e != NULL; e = e->next)
+	for (e = d->right; e != NULL; e = e->next)
 	{
 		i = pconnector_hash(dt, e, i);
 	}
