@@ -26,7 +26,6 @@
 #include <string.h>
 #include <stdbool.h>
 #include <errno.h>
-#include <externs.h>
 #include <time.h>
 
 #include "api-structures.h"
@@ -35,6 +34,7 @@
 #include "print/print-util.h" // For patch_subscript_mark()
 #include "dict-common/regex-morph.h"
 #include "error.h"
+#include "externs.h"
 #include "tokenize.h"
 #include "tok-structures.h"
 
@@ -385,9 +385,9 @@ bool anysplit_init(Dictionary afdict)
 	as->regmid = regbuild(regmid->string, regmid->length, AFDICT_REGMID);
 	as->regsuf = regbuild(regsuf->string, regsuf->length, AFDICT_REGSUF);
 
-	if (compile_regexs(as->regpre, NULL) != 0) return false;
-	if (compile_regexs(as->regmid, NULL) != 0) return false;
-	if (compile_regexs(as->regsuf, NULL) != 0) return false;
+	if (!compile_regexs(as->regpre, NULL) != 0) return false;
+	if (!compile_regexs(as->regmid, NULL) != 0) return false;
+	if (!compile_regexs(as->regsuf, NULL) != 0) return false;
 
 	as->nparts = atoi(regparts->string[0]);
 	if (as->nparts < 0)
