@@ -12,9 +12,7 @@
 /*************************************************************************/
 
 #include <ctype.h>
-#ifdef HAVE_POSIX_MEMALIGN
 #include <errno.h>
-#endif
 #include <limits.h>
 #ifdef _WIN32
 #define _CRT_RAND_S
@@ -35,6 +33,8 @@
 	#include <windows.h>
 #endif /* _WIN32 */
 
+#include "link-includes.h"
+#include "error.h"
 #include "utilities.h"
 
 /* This file contains general utilities that fix, enhance OS-provided
@@ -44,6 +44,48 @@
 /* ============================================================= */
 /* String utilities */
 
+/* Windows, POSIX and GNU have different ideas about strerror_r().  hence
+ * use our own function that uses the available system function and is
+ * consistent. It doesn't try to mimic exactly any version of
+ * strerror_r(). */
+#ifdef _WIN32
+void lg_strerror(int err_no, char *buf, size_t len)
+{
+	if (strerror_s(buf, len, err_no) != 0)
+		strerror_s(buf, len, errno); /* errno got set by previous strerror_s() */
+}
+#else
+#if HAVE_STRERROR_R
+
+#if STRERROR_R_CHAR_P
+/* Using the GNU version. */
+void lg_strerror(int err_no, char *buf, size_t len)
+{
+	char *errstr = strerror_r(err_no, buf, len);
+	strncpy(buf, errstr, len);
+	buf[len-1] = '\0';
+}
+#else /* !STRERROR_R_CHAR_P */
+/* Using the XSI version. */
+void lg_strerror(int err_no, char *buf, size_t len)
+{
+	errno = 0;
+	if ((strerror_r(err_no, buf, len) == EINVAL) || (errno == EINVAL))
+		snprintf(buf, len, "Unknown error %d", err_no);
+}
+#endif /* STRERROR_R_CHAR_P */
+
+#else /* !STRERROR_R */
+/* No strerror_r()??? No thread-safe error message - use a workaround.
+ * (FIXME Could check if threads are not supported and use strerror(),
+ * else protect strerror().) */
+void lg_strerror(int err_no, char *buf, size_t len)
+{
+	snprintf(buf, len, "Error %d", err_no);
+}
+#endif /* STRERROR_R */
+#endif /* _WIN32 */
+
 char *safe_strdup(const char *u)
 {
 	if (u)
@@ -521,6 +563,11 @@ const char * dyn_str_value(dyn_str* s)
 	return s->str;
 }
 
+size_t dyn_strlen(dyn_str* s)
+{
+	return s->end;
+}
+
 /* ======================================================== */
 /* Locale routines */
 
@@ -640,7 +687,7 @@ win32_getlocale (void)
 char * get_default_locale(void)
 {
 	const char *lc_vars[] = {"LC_ALL", "LC_CTYPE", "LANG", NULL};
-	char *ev;
+	char *ev = NULL;
 	const char **evname;
 	char *locale = NULL;
 
@@ -707,11 +754,12 @@ static void free_C_LC_NUMERIC(void)
 }
 #endif /* HAVE_LOCALE_T */
 
-bool strtodC(const char *s, double *r)
+/* FIXME: Rewrite to directly convert scaled integer strings (only). */
+bool strtodC(const char *s, float *r)
 {
 	char *err;
 
-#ifdef HAVE_LOCALE_T
+#if defined(HAVE_LOCALE_T) && !defined(__sun__) && !defined(__OpenBSD__)
 	double val = strtod_l(s, &err, get_C_LC_NUMERIC());
 #else
 	/* dictionary_setup_locale() invokes setlocale(LC_NUMERIC, "C") */
@@ -720,7 +768,7 @@ bool strtodC(const char *s, double *r)
 
 	if ('\0' != *err) return false; /* *r unaffected */
 
-	*r = val;
+	*r = (float)val;
 	return true;
 }
 
